@@ -2,19 +2,17 @@ package DBIx::Class::Core;
 
 use strict;
 use warnings;
-no warnings 'qw';
 
 use base qw/DBIx::Class/;
 
 __PACKAGE__->load_components(qw/
-  Serialize::Storable
-  InflateColumn
   Relationship
+  InflateColumn
   PK::Auto
   PK
   Row
   ResultSourceProxy::Table
-  AccessorGroup/);
+/);
 
 1;
 
@@ -24,8 +22,8 @@ DBIx::Class::Core - Core set of DBIx::Class modules
 
 =head1 SYNOPSIS
 
-  # In your table classes
-  __PACKAGE__->load_components(qw/Core/);
+  # In your result (table) classes
+  use base 'DBIx::Class::Core';
 
 =head1 DESCRIPTION
 
@@ -36,25 +34,26 @@ The core modules currently are:
 
 =over 4
 
-=item L<DBIx::Class::Serialize::Storable>
-
 =item L<DBIx::Class::InflateColumn>
 
-=item L<DBIx::Class::Relationship>
+=item L<DBIx::Class::Relationship> (See also L<DBIx::Class::Relationship::Base>)
+
+=item L<DBIx::Class::PK::Auto>
 
 =item L<DBIx::Class::PK>
 
 =item L<DBIx::Class::Row>
 
-=item L<DBIx::Class::ResultSourceProxy::Table>
-
-=item L<DBIx::Class::AccessorGroup>
+=item L<DBIx::Class::ResultSourceProxy::Table> (See also L<DBIx::Class::ResultSource>)
 
 =back
 
+A better overview of the methods found in a Result class can be found
+in L<DBIx::Class::ResultClass>.
+
 =head1 AUTHORS
 
-Matt S. Trout <mst@shadowcatsystems.co.uk>
+See L<DBIx::Class/CONTRIBUTORS>
 
 =head1 LICENSE
 
