@@ -1,4 +1,5 @@
-package DBIx::Class::PK::Auto::DB2;
+package # hide package from pause
+  DBIx::Class::PK::Auto::DB2;
 
 use strict;
 use warnings;
@@ -9,7 +10,7 @@ __PACKAGE__->load_components(qw/PK::Auto/);
 
 1;
 
-=head1 NAME 
+=head1 NAME
 
 DBIx::Class::PK::Auto::DB2 - (DEPRECATED) Automatic primary key class for DB2
 
@@ -19,7 +20,7 @@ Just load PK::Auto instead; auto-inc is now handled by Storage.
 
 =head1 AUTHORS
 
-Matt S Trout <mst@shadowcatsystems.co.uk>
+See L<DBIx::Class/CONTRIBUTORS>
 
 =head1 LICENSE
 
