@@ -1,4 +1,5 @@
-package DBIx::Class::PK::Auto::MSSQL;
+package # hide package from pause
+  DBIx::Class::PK::Auto::MSSQL;
 
 use strict;
 use warnings;
@@ -19,7 +20,7 @@ Just load PK::Auto instead; auto-inc is now handled by Storage.
 
 =head1 AUTHORS
 
-Matt S Trout <mst@shadowcatsystems.co.uk>
+See L<DBIx::Class/CONTRIBUTORS>
 
 =head1 LICENSE
 
