@@ -47,6 +47,7 @@ sub new {
 
   my $orig_attrs = $rs->_resolved_attrs;
   my $alias = $rs->current_source_alias;
+  my $rsrc = $rs->result_source;
 
   # If $column can be found in the 'as' list of the parent resultset, use the
   # corresponding element of its 'select' list (to keep any custom column
@@ -57,18 +58,24 @@ sub new {
   my $as_index = List::Util::first { ($as_list->[$_] || "") eq $column } 0..$#$as_list;
   my $select = defined $as_index ? $select_list->[$as_index] : $column;
 
-  my $new_parent_rs;
+  my ($new_parent_rs, $colmap);
+  for ($rsrc->columns, $column) {
+    if ($_ =~ /^ \Q$alias\E \. ([^\.]+) $ /x) {
+      $colmap->{$_} = $1;
+    }
+    elsif ($_ !~ /\./) {
+      $colmap->{"$alias.$_"} = $_;
+      $colmap->{$_} = $_;
+    }
+  }
+
   # analyze the order_by, and see if it is done over a function/nonexistentcolumn
   # if this is the case we will need to wrap a subquery since the result of RSC
   # *must* be a single column select
-  my %collist = map 
-    { $_ => 1, ($_ =~ /\./) ? () : ( "$alias.$_" => 1 ) }
-    ($rs->result_source->columns, $column)
-  ;
   if (
     scalar grep
-      { ! $collist{$_->[0]} }
-      ( $rs->result_source->schema->storage->_extract_order_criteria ($orig_attrs->{order_by} ) ) 
+      { ! exists $colmap->{$_->[0]} }
+      ( $rsrc->schema->storage->_extract_order_criteria ($orig_attrs->{order_by} ) )
   ) {
     # nuke the prefetch before collapsing to sql
     my $subq_rs = $rs->search;
@@ -89,24 +96,11 @@ sub new {
   # we need to group *IF WE CAN* (only if the column in question is unique)
   if (!$orig_attrs->{group_by} && keys %{$orig_attrs->{collapse}}) {
 
-    # scan for a constraint that would contain our column only - that'd be proof
-    # enough it is unique
-    my $constraints = { $rs->result_source->unique_constraints };
-    for my $constraint_columns ( values %$constraints ) {
-
-      next unless @$constraint_columns == 1;
-
-      my $col = $constraint_columns->[0];
-      my $fqcol = join ('.', $new_attrs->{alias}, $col);
-
-      if ($col eq $select or $fqcol eq $select) {
-        $new_attrs->{group_by} = [ $select ];
-        delete $new_attrs->{distinct}; # it is ignored when group_by is present
-        last;
-      }
+    if ($colmap->{$select} and $rsrc->_identifying_column_set([$colmap->{$select}])) {
+      $new_attrs->{group_by} = [ $select ];
+      delete $new_attrs->{distinct}; # it is ignored when group_by is present
     }
-
-    if (!$new_attrs->{group_by}) {
+    else {
       carp (
           "Attempting to retrieve non-unique column '$column' on a resultset containing "
         . 'one-to-many joins will return duplicate results.'
@@ -124,7 +118,7 @@ sub new {
 
 =item Arguments: none
 
-=item Return Value: \[ $sql, @bind ]
+=item Return Value: \[ $sql, L<@bind|DBI/Placeholders-and-Bind-Values> ]
 
 =back
 
@@ -149,7 +143,7 @@ sub as_query { return shift->_resultset->as_query(@_) }
 Returns the next value of the column in the resultset (or C<undef> if
 there is none).
 
-Much like L<DBIx::Class::ResultSet/next> but just returning the 
+Much like L<DBIx::Class::ResultSet/next> but just returning the
 one value.
 
 =cut
@@ -177,7 +171,7 @@ Returns all values of the column in the resultset (or C<undef> if
 there are none).
 
 Much like L<DBIx::Class::ResultSet/all> but returns values rather
-than row objects.
+than result objects.
 
 =cut
 
@@ -292,7 +286,7 @@ sub min {
 
 =item Arguments: none
 
-=item Return Value: $resultset
+=item Return Value: L<$resultset|DBIx::Class::ResultSet>
 
 =back
 
@@ -331,7 +325,7 @@ sub max {
 
 =item Arguments: none
 
-=item Return Value: $resultset
+=item Return Value: L<$resultset|DBIx::Class::ResultSet>
 
 =back
 
@@ -370,7 +364,7 @@ sub sum {
 
 =item Arguments: none
 
-=item Return Value: $resultset
+=item Return Value: L<$resultset|DBIx::Class::ResultSet>
 
 =back
 
@@ -419,7 +413,7 @@ sub func {
 
 =item Arguments: $function
 
-=item Return Value: $resultset
+=item Return Value: L<$resultset|DBIx::Class::ResultSet>
 
 =back
 
@@ -441,7 +435,7 @@ sub func_rs {
 
 See L<DBIx::Class::Schema/throw_exception> for details.
 
-=cut 
+=cut
 
 sub throw_exception {
   my $self=shift;
@@ -480,9 +474,7 @@ sub _resultset {
 
 =head1 AUTHORS
 
-Luke Saunders <luke.saunders@gmail.com>
-
-Jess Robinson
+See L<DBIx::Class/CONTRIBUTORS>
 
 =head1 LICENSE
 
